######################################
# Statistical methods for data science: R 
#
# http://didawiki.di.unipi.it/doku.php/mds/smd/ 
######################################

# [CTRL-Enter] to execute a line

###### Script Lesson 09: Moments, joint distributions, sum of random variables ##########

#### Moments

# positive skewness and platykurtic
x = rbinom(10000, 50, 0.1)
plot(table(x)/length(x), type='p')
mu = mean(x) # sample mean
sigma = sd(x) # sample sd
mu3 = mean((x-mu)^3)/sigma^3 # sample skewness
mu3
# or using a library
library(moments)
skewness(x)
kurtosis(x)

# set working dir
wdir = "C:/Users/Salvatore Ruggieri/Desktop/tmp"
setwd(wdir)
# Data from Human Mortality Database https://www.mortality.org (require registration)
#library(HMDHFDplus)
#mortalitycount = readHMDweb(CNTRY='ITA', item='Deaths_1x1', username='<username>', password='<pwd>')
#save(mortalitycount, file="smd09.Rdata")
load(file="smd09.Rdata")
View(mortalitycount)

# negative skewness and leptokurtic
year = 2015
ndeaths = mortalitycount[mortalitycount$Year==year,]$Total
ages = mortalitycount[mortalitycount$Year==year,]$Age
plot(ages, ndeaths)
x = rep(ages, ndeaths)
mean(x)
plot(table(x)/length(x), xlab="ages", ylab="pmf")
skewness(x)
kurtosis(x)


#### Joint distributions

# discrete random variable example
outer(1:6, 1:6, "+")
sumv = as.vector(outer(1:6, 1:6, "+"))
sumv
outer(1:6, 1:6, pmax)
maxv = as.vector(outer(1:6, 1:6, pmax))
maxv
joint.pmf = table(sumv, maxv)
joint.pmf

library(plot3D)
hist3D(z=joint.pmf/36, ticktype="detailed", space=0.1, shade=0.5)
image2D(joint.pmf/36)

# tabular method for calculating marginal distribution
# of sum
sum.marginal.pmf = apply(joint.pmf, 1, sum)
sum.marginal.pmf # same as
table(sumv)
# of max 
max.marginal.pmf = apply(joint.pmf, 2, sum)
max.marginal.pmf # same as
table(maxv)

joint.cdf = t(apply(apply(joint.pmf, 2, cumsum), 1, cumsum))
joint.cdf
image2D(joint.cdf/36)

# bivariate normal density
f = function(x, y) { 30/pi * exp(-50*x^2 - 50*y^2+80*x*y) }
# grid to plot 
x = seq(-0.4, 0.4, length= 100)
y = x
# sample values over the grid
z = outer(x, y, f)
# perspective Plot
persp(x,y,z, theta = -45, phi = 45, expand = 0.5, col = "lightblue", ticktype="detailed")

#### Sum of random variables

# E(n,l) = X_1 + ... + X_n where X_i ~ Exp(l) are i.i.d.
# Gam(n, l) generalize E(n, l) to non-integer n

# Euler's gamma(x) function extends factorial(x-1)
plot(1:10, factorial(1:10), log="y")
curve(gamma(x+1), add=T, col=2)

curve(dexp(x, 1), ylim=c(0,1), xlim=c(0, 10)) # lambda = 1
curve(dgamma(x, 1, 1), add=T, col=2)
curve(dgamma(x, 2, 1), add=T, col=3)
curve(dgamma(x, 5, 1), add=T, col=4)
# simulation
n = 100; l = 0.5
re = replicate(10000, sum(rexp(n, l)))
hist(re, breaks=60, freq=FALSE) # histogram of re
curve(dgamma(x, n, l), col=2, add=T, lwd=3) # theoretical DF


